﻿#region File Description
//-----------------------------------------------------------------------------
// MuScreenManager
// 
// ScreenManager.cs
//-----------------------------------------------------------------------------
#endregion

#region USING
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
#endregion USING

namespace MuScreenManager
{
    /// <summary>
    /// Core ScreenManager engine, used to control, manage and display multiple GameScreens for an XNA game
    /// </summary>
    public class ScreenManager : DrawableGameComponent
    {
                                                #region PROPERTIES

        /// <summary>
        /// List of current screens
        /// </summary>
        private List<Screen> screenList = new List<Screen>();

        /// <summary>
        /// List of screens that need to be removed from usage
        /// </summary>
        private List<Screen> screenToRemoveList = new List<Screen>();

        /// <summary>
        /// Reference to the initial game that created it all
        /// </summary>
        private Game game;
        public Game Game
        {
            get { return game; }
        }

        /// <summary>
        /// Global Spritebatch, allows for drawing of all elements under the same spriteBatch
        /// </summary>
        private SpriteBatch spriteBatch;

        /// <summary>
        /// Relative URI to the content directory of this game
        /// </summary>
        private string contentDir;
        public string ContentDir
        {
            get { return contentDir; }
        }

                                                #endregion PROPERTIES

                                                #region METHODS

        /// <summary>
        /// Initialize an instance of the ScreenManager
        /// </summary>
        /// <param name="game">Reference to the initial game that created it all</param>
        /// <param name="graphicsDevice">Game's generated GraphicsDevice</param>
        /// <param name="contentDir">Directory where the content from the Content project</param>
        public ScreenManager(Game game, GraphicsDevice graphicsDevice, string contentDir)
            : base(game)
        {
            //Store the required references
            this.game = game;
            spriteBatch = new SpriteBatch(graphicsDevice);
            this.contentDir = contentDir;
        }

        /// <summary>
        /// Update the ScreenManager and all screens within
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of the game time</param>
        public override void Update(GameTime gameTime)
        {
            //Update all the screens in the current screenlist
            foreach (var screen in screenList)
            {
                screen.Update(gameTime);
            }

            //Remove all the screens from current rotation in the removal list
            for (int i = 0; i < screenToRemoveList.Count; i++)
            {
                screenToRemoveList[i].UnloadContent();
                screenList.Remove(screenToRemoveList[i]);
            }

            //Clear the removl screen list
            screenToRemoveList.Clear();
        }

        /// <summary>
        /// Draw the screens within ScreenManager
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of the game time</param>
        public override void Draw(GameTime gameTime)
        {
            //Start the spritebatch
            spriteBatch.Begin();

            //Draw all the screens
            foreach (var screen in screenList)
            {
                screen.Draw(spriteBatch);
            }

            //Everything on all screens has been drawn, end spritebatch
            spriteBatch.End();
        }

                                                #endregion METHODS

                                                #region PUBLIC METHODS

        /// <summary>
        /// Add a screen to this ScreenManager
        /// </summary>
        /// <param name="screen">Screen to add to rotation</param>
        public void AddScreen(Screen screen)
        {
            screen.ScreenManager = this;    //Add this ScreenManager reference to the screen
            screen.OnInitialize();          //Initializes objects that couldn't be initialized in the constructor
            screen.LoadContent();           //Load the content for the screen
            screen.Initialize();            //Initialize anything that couldn't be initialized in the constructor
                                            //because of content needing loaded

            //Add screen to current list of screens
            screenList.Add(screen);
        }

        /// <summary>
        /// Remove the screen from active ScreenManager
        /// </summary>
        /// <param name="screen">Screen to remove from rotation</param>
        public void RemoveScreen(Screen screen)
        {
            //Add screen to removal list, for removal at end of update cycle
            screenToRemoveList.Add(screen);
        }

                                                #endregion PUBLIC METHODS
    }
}
